###############################################################################
## Arcadia Project
##
## Copyright (C) 2009-2010 Arcadia Project arcadia.videogame@gmail.com http://www.arcadiaproject.org/
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###############################################################################

import pygame
from pygame.locals import *
import os
from lib import FileManager
from lib.ControlManager import ControlMapper
from lib.SpriteManager import SpriteManager
from lib.SimpleSurface import SimpleSurface
from lib.CollisionManager import CollisionManager
from lib.Shadows import *
from game.gameover.GameOver import GameOver
from Test import Test
from Test2 import Test2

class Game:
    """Clase para heredar con ciertas funcionalidades para abstraer al programador de minijuegos"""
    def __init__(self, core):
        self.core = core
        self.controllableSprites = []
        self.itemsToRender = []
        self.core.players = 1 #1 por defecto
        self.core.gameMode = None #nada asignado por defecto
        self.started = False
        
        self.showplayers = []
        
        self.showplayers.append(SimpleSurface("resources/GUI/rojo_small.png"))
        self.showplayers.append(SimpleSurface("resources/GUI/amarillo_small.png"))
        self.showplayers.append(SimpleSurface("resources/GUI/celeste_small.png"))
        self.showplayers.append(SimpleSurface("resources/GUI/verde_small.png"))
		
        self.gameover = GameOver(self.core, (self.core.screen_width * 0.5, self.core.screen_height * 0.5))
		
    def startGame(self,name):
        """metodo que se inicializa al comenzar un minigame"""

        result = FileManager.xmlReader("minigames/"+name+"/info.xml")
            
        dicc = result[0]

        if dicc.has_key("info"):
            info = dicc["info"]

        if info.has_key("name"):
            title = info["name"]

        if info.has_key("description"):
            description = info["description"]

        players = self.core.players
        

        #background = pygame.Surface((self.core.screen_width, self.core.screen_height), SRCALPHA)
        #background.fill((0,0,0))
        #background.set_alpha(150)
        
        surface = SimpleSurface("resources/GUI/infolayer/infolayer.png")

        #nueva = add_shadow(surface.getSurface(), (2,2),1, 0.5)
        #surface.setSurface(nueva)

        self.core.font.selectCustomFont("kimbalt_", 24)
        title = self.core.font.renderText(title, surface.max_rect().width * 0.5 , 100, (0,0,0), "center", "top")

        self.core.font.selectCustomFont("kimbalt_", 14)
        desc = self.core.font.renderParagraph(description, 0, 0, surface.max_rect().width * 0.99, surface.max_rect().height * 0.99)

        #surface_to_blit = background
        surface_to_blit = surface.getSurface()
        surface_to_blit.blit(title.getSurface(), (title.max_rect().left, title.max_rect().top))
        surface_to_blit.blit(desc.getSurface(), (self.core.screen_width * 0.2, title.max_rect().top + title.max_rect().height + 20))
        surface.setSurface(surface_to_blit)

        if os.path.exists("minigames/"+name+"/instructions.png"):
            instructions = SimpleSurface("minigames/"+name+"/instructions.png")
            instructions.setRect(surface.max_rect().width * 0.5, surface.max_rect().height - surface.max_rect().height * 0.1, "center", "bottom")
            surface_to_blit.blit(instructions.getSurface(), (instructions.max_rect().left, instructions.max_rect().top))
        
        surface.setRect(self.core.screen_width * 0.5, self.core.screen_height * 0.5, "center", "center")
        self.startSurface = surface

        self.core.pause = True
        

            
    def gameOver(self):
        return self.gameover
        
    def paint(self):
        pass

    def logic(self):
        pass

    def bindSpriteToControls(self,sprite):
        self.controllableSprites.append(sprite)
    
    def unbindSpritesToControls(self):
        self.controllableSprites = []

    def controls(self):
        """rescatamos las funciones de mapeado general del juego y el resto lo delegamos al minijuego"""
        if self.core.controlManager.controlSimple.count(("KEYBUTTON", pygame.K_RETURN)) > 0 and self.core.pause == True:                            
            
            self.core.pause = False

            self.core.controlManager.controlSimple.pop(self.core.controlManager.controlSimple.index(("KEYBUTTON", pygame.K_RETURN)))
            self.startSurface.hide()
            self.started = True

        if self.started == True:

            if self.core.controlManager.controlEvents.count(("KEYBUTTON", pygame.K_q)) > 0:
                self.core.controlManager.running = False
            
            if self.core.controlManager.controlSimple.count(("KEYBUTTON", pygame.K_ESCAPE)) > 0:                            
  
                self.core.resumed = self.core.game
                self.core.game = self.core.games[5]
                self.core.controlManager.controlSimple.pop(self.core.controlManager.controlSimple.index(("KEYBUTTON", pygame.K_ESCAPE)))

            else:
                if self.core.pause == False:
                    controlMapper = ControlMapper(self.core.controlManager.controlEvents)
                    for sprite in self.controllableSprites:
                        sprite.controls(controlMapper)

    def paintItem(self, item):
        """agrega el sprite a la lista de elementos a pintar por el render"""
        if isinstance(item, dict):
            self.core.renderManager.paintGUI(item)
        else:
            if item.isPrintable():
                self.core.renderManager.paintItem(item)
		
    def setPlayers(self, num):
        """define el numero de jugadores que participaran"""
        self.core.players = num

    def setGameMode(self, mode):
        """define el modo de juego a ejecutar"""
        self.core.gameMode = mode
        
    def paintElements(self):
        #imprimimos la GUI de los jugadores siempre encima
        for player in range(0, self.core.players - 1):
			self.showplayers[player].setRect(self.core.screen_width * 0.01 + player * (self.showplayers[player].max_rect().width + 20) , self.core.screen_height * 0.90)
			self.paintItem(self.showplayers[player])

