###############################################################################
## Arcadia Project
##
## Copyright (C) 2009-2010 Arcadia Project arcadia.videogame@gmail.com http://www.arcadiaproject.org/
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###############################################################################

import pygame
import os, sys
from lib.ConfigManager import ConfigHandler
from Game import Game
from game.intro.Fondo import Fondo
from lib.Shadows import *
from lib.SimpleSurface import SimpleSurface
from game.configMenu.elements.Selector import Selector
from lib import FileManager

class ConfigMenu(Game):

    def __init__(self, core):

        Game.__init__(self,core)
        self.core = core
        self.core.font.selectCustomFont("kimbalt_", 30)


        #INFORMATION
        self.titulo = self.core.font.renderText("Configuracion:", self.core.screen_width * 0.01, self.core.screen_height * 0.05, (0,153,255))
        sombra = add_shadow(self.titulo.getSurface(), (2,2))
        self.titulo.setSurface(sombra)

        #self.fondo = Fondo(self.core, (0,0))
        #self.fondo2 = Fondo(self.core, (self.core.screen_width, 0))
        self.selector = Selector()

        self.barra_superior = SimpleSurface("game"+os.path.sep+"configMenu"+os.path.sep+"images"+os.path.sep+"barra_superior"+os.path.sep+"barra_superior.png")



        #FONTS
        self.core.font.selectCustomFont("kimbalt_", 48)

        color=(255,255,255)
        self.video = self.core.font.renderText("Video", self.core.screen_width * 0.2, self.core.screen_height * 0.3, color)
        sombra = add_shadow(self.video.getSurface(), (2,2))
        self.video.setSurface(sombra)

        self.selectedVideo = self.core.font.renderText("Video", self.core.screen_width * 0.2, self.core.screen_height * 0.3, (255,0,0))
        sombra = add_shadow(self.selectedVideo.getSurface(), (2,2))
        self.selectedVideo.setSurface(sombra)


        self.sound = self.core.font.renderText("Sonido", self.core.screen_width * 0.2, self.core.screen_height * 0.4, color)
        sombra = add_shadow(self.sound.getSurface(), (2,2))
        self.sound.setSurface(sombra)

        self.selectedSound = self.core.font.renderText("Sonido", self.core.screen_width * 0.2, self.core.screen_height * 0.4, (255,0,0))
        sombra = add_shadow(self.selectedSound.getSurface(), (2,2))
        self.selectedSound.setSurface(sombra)



        self.control = self.core.font.renderText("Controles", self.core.screen_width * 0.2, self.core.screen_height * 0.5, color)
        sombra = add_shadow(self.control.getSurface(), (2,2))
        self.control.setSurface(sombra)

        self.selectedControl = self.core.font.renderText("Controles", self.core.screen_width * 0.2, self.core.screen_height * 0.5, (255,0,0))
        sombra = add_shadow(self.selectedControl.getSurface(), (2,2))
        self.selectedControl.setSurface(sombra)



        self.selecc = self.core.font.renderText("Seleccionar Configuracion", self.core.screen_width * 0.2, self.core.screen_height * 0.7, color)
        sombra = add_shadow(self.selecc.getSurface(), (2,2))
        self.selecc.setSurface(sombra)

        self.seleccS = self.core.font.renderText("Seleccionar Configuracion", self.core.screen_width * 0.2, self.core.screen_height * 0.7, (255,0,0))
        sombra = add_shadow(self.seleccS.getSurface(), (2,2))
        self.seleccS.setSurface(sombra)

        self.core.font.selectCustomFont("kimbalt_", 20)

        self.actConfig = self.core.font.renderText("Configuracion Actual:", self.core.screen_width * 0.6, self.core.screen_height * 0.2, (0,0,255))
        sombra = add_shadow(self.actConfig.getSurface(), (1,1))
        self.actConfig.setSurface(sombra)

        #MENU 
        self.selected=0
        self.options=3
        self.controlMapping=False
        self.numPlayer=0
        self.mode=0 #0 -> Menu: Video, Sound, Controls
                    #10 -> Video
                    #11 -> Sonido
                    #12 -> Control
                    #13 -> Modificar Config

        
    def paint(self):
        self.paintItem(self.core.fondo)
        self.paintItem(self.core.fondo2)
        self.paintItem(self.barra_superior)
        self.paintItem(self.titulo)


        #MENU: Video-Sound-Controls-Select Config
        if self.mode==0:
            self.paintMainMenu()

        elif self.mode==10:
            #self.paintVideoMenu()
            
            if self.core.configManager.actualConfig.videoFullScreen:
                fS="Si"
            else:
                fS="No"
            self.paintMenu(elements=["Pantalla Completa","Resolucion"],values=[fS,str(self.core.configManager.actualConfig.videoResolutionX)+" x "+str(self.core.configManager.actualConfig.videoResolutionY)])

        elif self.mode==11:
            #self.paintSoundMenu()
            self.paintMenu(elements=["Volumen general","Volumen de sonidos","Volumen de musica"],values=[str(self.core.configManager.actualConfig.volumeGeneral),str(self.core.configManager.actualConfig.volumeSound),str(self.core.configManager.actualConfig.volumeMusic)])
            
        elif self.mode==12:

            #self.paintControlMenu()      
            self.paintMenu(elements=["  Jugador:","Arriba","Abajo","Izquierda","Derecha","Boton 1","Boton 2","Boton 3","Boton 4"],values=[str(self.numPlayer+1)]+self.core.configManager.actualConfig.controls[self.numPlayer].getButtonList())


        elif self.mode==13:
            self.pintarListaConfigs()
            self.paintItem(self.actConfig)
            confActual=self.core.font.renderText(self.core.configManager.actualConfig.name, self.core.screen_width * 0.6, self.core.screen_height * 0.25, (255,255,255))
            self.paintItem(confActual)
            #self.paintSelector()
            
        else:
            pass


    def logic(self):
        self.core.fondo.ia()
        self.core.fondo2.ia()
        




    def controls(self):


        if self.controlMapping and len(self.core.controlManager.controlSimple)>0:
            tmp=self.core.controlManager.controlSimple[len(self.core.controlManager.controlSimple)-1]
            if tmp[1]<>13:
                self.core.controlManager.controlSimple.pop(len(self.core.controlManager.controlSimple)-1)

                if self.selected==1:
                    self.core.configManager.actualConfig.controls[self.numPlayer].up=tmp[1]
                elif self.selected==2:
                    self.core.configManager.actualConfig.controls[self.numPlayer].down=tmp[1]
                elif self.selected==3:
                    self.core.configManager.actualConfig.controls[self.numPlayer].left=tmp[1]
                elif self.selected==4:
                    self.core.configManager.actualConfig.controls[self.numPlayer].right=tmp[1]
                elif self.selected==5:
                    self.core.configManager.actualConfig.controls[self.numPlayer].button1=tmp[1]
                elif self.selected==6:
                    self.core.configManager.actualConfig.controls[self.numPlayer].button2=tmp[1]
                elif self.selected==7:
                    self.core.configManager.actualConfig.controls[self.numPlayer].button3=tmp[1]
                elif self.selected==8:
                    self.core.configManager.actualConfig.controls[self.numPlayer].button4=tmp[1]

                self.controlMapping=False


        #CONTROLES ABSOLUTOS
        if self.core.controlManager.controlEvents.count(("KEYBUTTON", pygame.K_f)) > 0:
            self.core.toggleFullScreen()

        
        #CONTROLES MOVIMIENTO
        if self.core.controlManager.controlSimple.count(("KEYBUTTON", pygame.K_UP)) > 0:
            try:
                self.core.controlManager.controlSimple.pop(self.core.controlManager.controlSimple.index(("KEYBUTTON", pygame.K_UP)))
            except ValueError:
                pass 
            if self.options<>0:
                self.selected=(self.selected-1)%self.options

    
        if self.core.controlManager.controlSimple.count(("KEYBUTTON", pygame.K_DOWN)) > 0:
            try:
                self.core.controlManager.controlSimple.pop(self.core.controlManager.controlSimple.index(("KEYBUTTON", pygame.K_DOWN)))
            except ValueError:
                pass
            if self.options<>0:
                self.selected=(self.selected+1)%self.options

        #CONTROLES PARA CAMBIOS - CONTEXTUALES

        if self.core.controlManager.controlSimple.count(("KEYBUTTON", pygame.K_LEFT)) > 0:


            if self.mode==10:
                if self.selected==0:
                    self.core.configManager.actualConfig.prevFullScreen()
                elif self.selected==1:
                    self.core.configManager.actualConfig.prevResolution()
            elif self.mode==11:
                if self.selected==0:
                    self.core.configManager.actualConfig.lessVolumeGeneral()
                if self.selected==1:
                    self.core.configManager.actualConfig.lessVolumeSound()
                if self.selected==2:
                    self.core.configManager.actualConfig.lessVolumeMusic()
            elif self.mode==12:
                if self.selected==0:
                    self.numPlayer=(self.numPlayer-1)%4
            elif self.mode==13:
                pass
            else:
                pass


            self.core.controlManager.controlSimple.pop(self.core.controlManager.controlSimple.index(("KEYBUTTON", pygame.K_LEFT)))


        if self.core.controlManager.controlSimple.count(("KEYBUTTON", pygame.K_RIGHT)) > 0:

            if self.mode==10:
                if self.selected==0:
                    self.core.configManager.actualConfig.nextFullScreen()
                elif self.selected==1:
                    self.core.configManager.actualConfig.nextResolution()
            elif self.mode==11:
                if self.selected==0:
                    self.core.configManager.actualConfig.moreVolumeGeneral()
                if self.selected==1:
                    self.core.configManager.actualConfig.moreVolumeSound()
                if self.selected==2:
                    self.core.configManager.actualConfig.moreVolumeMusic()
            elif self.mode==12:
                if self.selected==0:
                    self.numPlayer=(self.numPlayer+1)%4
            elif self.mode==13:
                pass
            else:
                pass


            self.core.controlManager.controlSimple.pop(self.core.controlManager.controlSimple.index(("KEYBUTTON", pygame.K_RIGHT)))


        #CONTROLES CONTEXTUALES ---- RETURN
        
        if self.core.controlManager.controlSimple.count(("KEYBUTTON", pygame.K_RETURN)) > 0 and self.selected == 0 and self.mode==0:
            self.mode=10
            self.options=2
            self.selected=0
            self.core.controlManager.controlSimple.pop(self.core.controlManager.controlSimple.index(("KEYBUTTON", pygame.K_RETURN)))

        if self.core.controlManager.controlSimple.count(("KEYBUTTON", pygame.K_RETURN)) > 0 and self.selected == 1 and self.mode==0:
            self.mode=11
            self.options=3
            self.selected=0
            self.core.controlManager.controlSimple.pop(self.core.controlManager.controlSimple.index(("KEYBUTTON", pygame.K_RETURN)))

        if self.core.controlManager.controlSimple.count(("KEYBUTTON", pygame.K_RETURN)) > 0 and self.selected == 2 and self.mode==0:
            self.mode=12
            self.options=9
            self.selected=0
            self.core.controlManager.controlSimple.pop(self.core.controlManager.controlSimple.index(("KEYBUTTON", pygame.K_RETURN)))

        if self.core.controlManager.controlSimple.count(("KEYBUTTON", pygame.K_RETURN)) > 0 and self.selected == 3 and self.mode==0:
            self.mode=13
            if len(self.core.configManager.configs)>0:
                self.options=len(self.core.configManager.configs)

            else:
                self.options=0

            self.selected=0

            find=self.core.configManager.findConfigByName(self.core.configManager.actualConfig.name)
            if find>=0:
                self.selected=find

            self.core.controlManager.controlSimple.pop(self.core.controlManager.controlSimple.index(("KEYBUTTON", pygame.K_RETURN)))

        if self.core.controlManager.controlSimple.count(("KEYBUTTON", pygame.K_RETURN)) > 0 and self.mode==12:

            if self.controlMapping==False and self.selected<>0:
                self.controlMapping=True
            self.core.controlManager.controlSimple.pop(self.core.controlManager.controlSimple.index(("KEYBUTTON", pygame.K_RETURN)))

        if self.core.controlManager.controlSimple.count(("KEYBUTTON", pygame.K_RETURN)) > 0 and self.mode==13:
            self.core.configManager.setActualConfigByPos(self.selected)
            self.core.controlManager.controlSimple.pop(self.core.controlManager.controlSimple.index(("KEYBUTTON", pygame.K_RETURN)))


        if self.core.controlManager.controlSimple.count(("KEYBUTTON", pygame.K_RETURN)) > 0:
            self.core.controlManager.controlSimple.pop(self.core.controlManager.controlSimple.index(("KEYBUTTON", pygame.K_RETURN)))

        #CONTROLES CONTEXTUALES ---- ESCAPE

        if self.core.controlManager.controlSimple.count(("KEYBUTTON", pygame.K_ESCAPE)) > 0:
            if self.mode==0:

                if self.core.controlManager.controlSimple.count(("KEYBUTTON", pygame.K_LEFT)) > 0:
                    self.core.controlManager.controlSimple.pop(self.core.controlManager.controlSimple.index(("KEYBUTTON", pygame.K_LEFT)))
                if self.core.controlManager.controlSimple.count(("KEYBUTTON", pygame.K_RIGHT)) > 0:
                    self.core.controlManager.controlSimple.pop(self.core.controlManager.controlSimple.index(("KEYBUTTON", pygame.K_RIGHT)))

                
                string=self.core.configManager.exportFileConfig([self.core.configManager.actualConfig]+self.core.configManager.configs)
                #FileManager.fileWriter("config/general/configresult.xml",string)
                FileManager.fileWriter("config/general/config.xml",string)
                self.core.game = self.core.games[0]
        
            elif self.mode==10:
                self.mode=0
                self.options=4
                self.selected=0
            elif self.mode==11:
                self.mode=0
                self.options=4
                self.selected=1
            elif self.mode==12:
                self.mode=0
                self.options=4
                self.selected=2
            elif self.mode==13:
                self.mode=0
                self.options=4
                self.selected=3

            else:
                pass

            self.core.controlManager.controlSimple.pop(self.core.controlManager.controlSimple.index(("KEYBUTTON", pygame.K_ESCAPE)))

    #ASSISTANCE


    def pintarListaConfigs(self):

        paintConfigs=[]

        pos=0.25
        i=0
        color=(0,0,0)

        lista=[]
        for e in self.core.configManager.configs:
            lista.append(e.name)

        self.core.font.selectCustomFont("kimbalt_", 30)

        for nom in lista:
            pos=pos+0.05
    
            if nom==self.core.configManager.actualConfig.name and i==self.selected:
                color=(100,100,255)           
            elif nom==self.core.configManager.actualConfig.name:
                color=(0,0,155)
            elif i==self.selected:
                color=(155,155,155)
            else:
                color=(0,0,0)
            paintConfigs.append(self.core.font.renderText(nom, self.core.screen_width * 0.20, self.core.screen_height * pos, color))
            i=i+1
        for e in paintConfigs:
            self.paintItem(e)


    def paintMainMenu(self):

            self.options=4
            if self.selected==0:
                self.paintItem(self.selectedVideo)
            else:
                self.paintItem(self.video)

            if self.selected==1:
                self.paintItem(self.selectedSound)
            else:
                self.paintItem(self.sound)

            if self.selected==2:
                self.paintItem(self.selectedControl)
            else:
                self.paintItem(self.control)

            if self.selected==3:
                self.paintItem(self.seleccS)
            else:
                self.paintItem(self.selecc)


    def paintMenu(self, elements=[], values=[], limitX=(0.2,0.6), limitY=(0.3,0.8)):

        self.core.font.selectCustomFont("kimbalt_", 30)

        els=len(elements)
        paintList=[]
        if els<>len(values) and els<=10:
            return 0
        else:
            i=0
            coordX=limitX[0]
            coordY=limitY[0]
            while ((i>=0) and (i<els)):
                
                coordY=limitY[0]+(limitY[1]-limitY[0])/els*i

                colorA=(0,150,255)
                if self.selected==i:
                    if self.controlMapping:
                        colorA=(255,255,0)
                    else:
                        colorA=(255,0,0)

                self.text = self.core.font.renderText(elements[i], self.core.screen_width * coordX, self.core.screen_height * coordY, colorA)
                sombra = add_shadow(self.text.getSurface(), (2,2))
                self.text.setSurface(sombra)

                colorB=(255,255,255)                
                paintList.append(self.text)
                if self.selected==i:
                    if self.controlMapping:
                        colorB=(255,255,0)
                    else:
                        colorB=(255,255,255)                


                self.textValue = self.core.font.renderText(values[i], self.core.screen_width * ((coordX+0.4)%1.0), self.core.screen_height * coordY, colorB)
                sombra = add_shadow(self.textValue.getSurface(), (2,2))
                self.textValue.setSurface(sombra)

                paintList.append(self.textValue)
                i=i+1

            for e in paintList:
                self.paintItem(e)


    def paintSelector(self):

            if self.mode==13:
                self.selector.rect.top=0.05*self.selected
            else:
                self.selector.rect.top=0.3+(0.5/self.options)*self.selected

            self.paintItem(self.selector)



