###############################################################################
## Arcadia Project
##
## Copyright (C) 2009-2010 Arcadia Project arcadia.videogame@gmail.com http://www.arcadiaproject.org/
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###############################################################################

class AnimationManager:
    """Clase encargada de encapsular una secuencia de imagenes para hacer una animacion"""

    def __init__(self, sequence, name):
        """Inicializacion de parametros por defecto"""
        self.sequence = sequence
        self.name = name
        self.cursor = 0
        self.animationQueue = []
        self.printable = True
        self.pending_printable = True

    def getAnimation(self):
        """Devuelve la lista con la secuencia de imagenes"""
        return self.sequence

    def getName(self):
        """Devuelve el nombre de la animacion"""
        return self.name

    def getTotalImages(self, frames):
        """Cuenta numero total de frames de la animacion"""
        return len(frames)

    def addFrameToQueue(self):
        """Obtiene la imagen de la animacion a devolver y actualiza el cursor para devolver otra imagen en la siguiente llamada""" 
        current = self.cursor

        #comprobamos antes si el sprite puede imprimirse
        if current == 0 and self.pending_printable == False:
            self.printable = False

        self.cursor += 1

        if self.cursor >= self.getTotalImages(self.sequence):
            self.cursor = 0
        if len(self.animationQueue) == 0:
            self.animationQueue.append(self.sequence[current])

    def getFrame(self):
        if len(self.animationQueue) == 0: #si esta vacia la animacion devolvemos un frame por defecto
            return self.sequence[0]
        else:
            return self.animationQueue.pop(0)
        
