###############################################################################
## Arcadia Project
##
## Copyright (C) 2009-2010 Arcadia Project arcadia.videogame@gmail.com http://www.arcadiaproject.org/
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###############################################################################

import pygame
from pygame.locals import *

class CollisionManager(object):
    """clase con metodos para tratar las colisiones del juego"""

    def __init__(self):
        pass

    def perfectCollision(self, sprite1, sprite2):
        """Tratamiento de la colision creando una mascara en blanco y negro para ver la colision pixel por pixel"""
        return pygame.sprite.collide_mask(sprite1, sprite2)

    def edgeCollision(self, sprite1, sprite2):
        """Metodo que detecta los lados de la colision"""        
        edges = []
        rect1 = sprite1.get_rect()
        rect2 = sprite2.get_rect()

        if self.perfectCollision(sprite1, sprite2):
            """Ha colisionado con la mascara y ahora vamos a ver los lados por donde ha colisionado"""
            
            #colision por la derecha
            if (rect2.left - rect1.left) > 0:
                edges.append("right")
            #colision por la izquierda
            if (rect2.left - rect1.left) < 0:
                edges.append("left")
            #colision por abajo
            if (rect2.top - rect1.top) > 0:
                edges.append("down")
            #colision por arriba
            if (rect2.top - rect1.top) < 0:
                edges.append("up")

        return edges

    def rectangleCollision(self, sprite1, sprite2):
        """metodo que encapsula la funcion de colision por rectangulos de pygame"""
        return pygame.sprite.collide_rect(sprite1, sprite2)

    def circleCollision(self, sprite1, sprite2):
        """metodo que encapsula la funcion de colision por circulos de pygame"""
        return pygame.sprite.collide_circle()

    def groupPerfectCollision(self, sprite1, group):
        
        collision = False

        for sprite2 in group:
            collision = collision or self.perfectCollision(sprite1, sprite2)

        return collision
        
    def groupEdgeCollision(self, sprite1, group):
        
        collision = False

        for sprite2 in group:
            collision = collision or self.edgeCollision(sprite1, sprite2)

        return collision


    
