###############################################################################
## Arcadia Project
##
## Copyright (C) 2009-2010 Arcadia Project arcadia.videogame@gmail.com http://www.arcadiaproject.org/
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###############################################################################

import FileManager
import pygame

class ConfigHandler:
    

    def __init__(self):

        self.defaultConfig=ConfigHandler.Config()
        self.actualConfig=None

        self.configs=[]
    

        self.importConfig("config/general/config.xml")
        self.setActualConfigByPos(0)

        if self.actualConfig==None:
            self.actualConfig=self.Config()




    #---------- IMPORT -----------

    def importConfig(self,path):

        self.configs=[]
        result=FileManager.xmlReader(path)
        if result==-1:
            #TODO: Imprimir por pantalla
            print "ConfigManager.importConfig -> Archivo no encontrado"
            return -1
        else:
            dicc=result[0]
            if dicc.has_key("configfile"):
                for e in dicc["configfile"].items():
                    self.configs.append(self.createConfigs(e))

    def createConfigs(self,tuples):

        #TODO: Siempre que sea publico o accesible desde aqui
        config=ConfigHandler.Config()

        config.name=tuples[0]

        try: 
            if self.stdConfigName==None:
                self.stdConfigName=config.name
        except AttributeError:
            self.stdConfigName=config.name

        confDicc=tuples[1]

        if confDicc.has_key("video"):
            tmp=confDicc["video"]

            if tmp.has_key("fullscreen"):
                try:
                    config.videoFullScreen=bool(tmp["fullscreen"])
                except NameError:
                    config.setSTDvideoFullScreen()
            if tmp.has_key("resolutionX"):
                try:
                    config.videoResolutionX=int(tmp["resolutionX"])
                except ValueError:
                    config.setSTDvideoResolutionX()

            if tmp.has_key("resolutionY"):
                try:
                    config.videoResolutionY=int(tmp["resolutionY"])
                except ValueError:
                    config.setSTDvideoResolutionY()


        if confDicc.has_key("volume"):
            tmp=confDicc["volume"]

            if tmp.has_key("general"):
                try:
                    config.volumeGeneral=int(tmp["general"])
                except ValueError:
                    config.setSTDvolumeGeneral()

            if tmp.has_key("sounds"):
                try:
                    config.volumeSound=int(tmp["sounds"])
                except ValueError:
                    config.setSTDvolumeSound()
            if tmp.has_key("music"):
                try:
                    config.volumeMusic=int(tmp["music"])
                except ValueError:
                    config.setSTDvolumeMusic()

        if confDicc.has_key("control"):
            tmp=confDicc["control"]      
            if tmp.has_key("P1"):
                config.controls[0]=self.createControl(tmp["P1"])
            if tmp.has_key("P2"):
                config.controls[1]=self.createControl(tmp["P2"])
            if tmp.has_key("P3"):
                config.controls[2]=self.createControl(tmp["P3"])
            if tmp.has_key("P4"):
                config.controls[3]=self.createControl(tmp["P4"])

        #TODO
        config.validateConfig()
        return config



    def createControl(self,controlScheme):
        
        actControl=ConfigHandler.Control()
        if controlScheme.has_key("up"):
            try:
                actControl.up=int(controlScheme["up"])
            except ValueError:
                actControl.up=0

        if controlScheme.has_key("down"):
            try:
                actControl.down=int(controlScheme["down"])
            except ValueError:
                actControl.down=0

        if controlScheme.has_key("left"):
            try:
                actControl.left=int(controlScheme["left"])
            except ValueError:
                actControl.left=0

        if controlScheme.has_key("right"):
            try:
                actControl.right=int(controlScheme["right"])
            except ValueError:
                actControl.right=0


        if controlScheme.has_key("button1"):
            try:
                actControl.button1=int(controlScheme["button1"])
            except ValueError:
                actControl.button1=0

        if controlScheme.has_key("button2"):
            try:
                actControl.button2=int(controlScheme["button2"])
            except ValueError:
                actControl.button2=0

        if controlScheme.has_key("button3"):
            try:
                actControl.button3=int(controlScheme["button3"])
            except ValueError:
                actControl.button3=0

        if controlScheme.has_key("button4"):
            try:
                actControl.button4=int(controlScheme["button4"])
            except ValueError:
                actControl.button4=0

        return actControl





    #---------- EXPORT -----------

    def exportFileConfig(self,configList):
        
        string="<configfile>\n\t"

        for conf in configList:
            string=string+self.exportConfig(conf)

        string=string+"\n</configfile>"

        return string

    def exportConfig(self,config):

        stringconf="\n\t<"+config.name+">"

        stringconf=stringconf+self.exportVolume(config)
        stringconf=stringconf+self.exportVideo(config)
        stringconf=stringconf+self.exportControl(config.controls)

        stringconf=stringconf+"\n\t</"+config.name+">"

        return stringconf


    def exportVolume(self,config):
    
        volGen="\n\t\t\t\t<general>"+str(config.volumeGeneral)+"</general>"
        volSnd="\n\t\t\t\t<sounds>"+str(config.volumeSound)+"</sounds>"
        volMus="\n\t\t\t\t<music>"+str(config.volumeMusic)+"</music>"
        
        return "\n\t\t\t<volume>"+volGen+volSnd+volMus+"\n\t\t\t</volume>"


    def exportVideo(self,config):
        
        vidResX="\n\t\t\t\t<resolutionX>"+str(config.videoResolutionX)+"</resolutionX>"
        vidResY="\n\t\t\t\t<resolutionY>"+str(config.videoResolutionY)+"</resolutionY>"
        fullscreen="\n\t\t\t\t<fullscreen>"+str(config.videoFullScreen)+"</fullscreen>"
        
        return "\n\t\t\t<video>"+vidResX+vidResY+fullscreen+"\n\t\t\t</video>"



    def exportControl(self,controls):

        string="\n\t\t\t<control>"

        i=1
        for e in controls:
            string=string+e.export(i)
            i=i+1

        string=string+"\n\t\t\t</control>"
        return string



    #-------MANAGER-------

    def setActualConfigByName(self,name):
        self.setActualConfigByPos(self,self.findConfigByName(self,name))

    def setActualConfigByPos(self,pos):
        if self.actualConfig<>None:
            self.configs.append(self.actualConfig)
        try:
            self.actualConfig=self.configs.pop(pos)
        except IndexError:
            return -1

    def eliminateByName(self,name):
        self.controls.pop(self.findConfigByName(name))
        
    def findConfigByName(self,name):
        i=0
        result=i
        for e in self.configs:
            if name==e.name:
               result=i
               break
            else:
               pass

            if len(self.configs)>0:
               i=(i+1)%len(self.configs)
            else:
               i=i+1

        if i==result:
            return result
        else:
            return -1
    
    def getNameConfigs(self):
        listConf=[]
        for e in self.configs:
            listConf.append(e.name)
        return listConf



    def getKey(self,numPlayer,buttonname):

        numPlayer=numPlayer-1
        numPlayer=numPlayer%4
        
        control=self.actualConfig.controls[numPlayer]
        
        if buttonname=="UP" or buttonname=="up":
            return control.up
        if buttonname=="DOWN" or buttonname=="down":
            return control.down
        if buttonname=="LEFT" or buttonname=="left":
            return control.left
        if buttonname=="RIGHT" or buttonname=="right":
            return control.right
        if buttonname=="BUTTON1" or buttonname=="button1":
            return control.button1
        if buttonname=="BUTTON2" or buttonname=="button2":
            return control.button2
        if buttonname=="BUTTON3" or buttonname=="button3":
            return control.button3
        if buttonname=="BUTTON4" or buttonname=="button4":
            return control.button4


    def getKeyName(self,numPlayer,buttonname):
        return pygame.key.name(self.getKey(numPlayer,buttonname))





    #---------- CONFIG CLASS -----------

    class Config (object):
    
        def __init__(self,nameC="sintitulo",vidResX=1024,vidResY=768,vidFS=False,volGen=5,volSnd=5,volMus=5):
    
            self.name=str(nameC)
            self.videoResolutionX=int(vidResX)
            self.videoResolutionY=int(vidResY)
            self.videoFullScreen=bool(vidFS)

            self.volumeGeneral=int(volGen) #0 to 10
            self.volumeSound=int(volSnd)  #0 to 10
            self.volumeMusic=int(volMus)  #0 to 10

            self.validResolutions=[(1024,768),(1024,800),(1280,768),(1280,800)]
            self.validVolumes=[0,1,2,3,4,5,6,7,8,9,10]            

            self.controls=[ConfigHandler.Control(),ConfigHandler.Control(),ConfigHandler.Control(),ConfigHandler.Control()]


        def validateConfig(self):
            modified=False

            if self.validatevideoResolution()==False:
                self.setSTDvideoResolution()
                modified=(modified or True)

            if self.validatevideoFullScreen()==False:
                self.setSTDvideoFullScreen()
                modified=(modified or True)

            if self.validatevolumeGeneral()==False:
                self.setSTDvolumeGeneral()
                modified=(modified or True)
            if self.validatevolumeSound()==False:
                self.setSTDvolumeSound()
                modified=(modified or True)
            if self.validatevolumeMusic()==False:
                self.setSTDvolumeMusic()
                modified=(modified or True)

            for e in self.controls:
                e.validateControl()

            return modified


        def validatevideoResolution(self):
            px=self.videoResolutionX
            py=self.videoResolutionY
            tuple1=(px,py)
            try:
                if self.validResolutions.index(tuple1)>-1:
                    return True
            except ValueError:
                return False


        def validatevideoResolutionX(self):
            values=[1280,1024]
            px=self.videoResolutionX
            try:
                if values.index(px)>-1:
                    return True
            except ValueError:
                return False

        def validatevideoResolutionY(self):
            values=[800,768]
            px=self.videoResolutionY
            try:
                if values.index(px)>-1:
                    return True
            except ValueError:
                return False

        def validatevideoFullScreen(self):
            val=self.videoFullScreen
            if val==True or val==False:
                return True
            else:
                return False

        def validateVolume(self,volint):
            if type(volint)==type(0):
                if volint>=0 and volint<=10:
                    return True
                else:
                    return False
            else:
                return False

        def validatevolumeGeneral(self):
            self.validateVolume(self.volumeGeneral)
        def validatevolumeSound(self):
            self.validateVolume(self.volumeSound)
        def validatevolumeMusic(self):
            self.validateVolume(self.volumeMusic)


        def setSTDvideoResolution(self):
            self.videoResolutionX=self.validResolutions[0][0]
            self.videoResolutionY=self.validResolutions[0][1]
        def setSTDvideoResolutionX(self):
            self.videoResolutionX=self.validResolutions[0][0]
        def setSTDvideoResolutionY(self):
            self.videoResolutionY=self.validResolutions[0][1]
        def setSTDvideoFullScreen(self):
            self.videoFullScreen=False
        def setSTDvolumeGeneral(self):
            self.volumeGeneral=5
        def setSTDvolumeSound(self):
            self.volumeSound=5
        def setSTDvolumeMusic(self):
            self.volumeMusic=5

        def prevResolution(self):        
            try:
                val=self.validResolutions.index((self.videoResolutionX,self.videoResolutionY))
                if val>0:
                    self.videoResolutionX=self.validResolutions[val-1][0]
                    self.videoResolutionY=self.validResolutions[val-1][1]
                else:
                    self.videoResolutionX=self.validResolutions[0][0]
                    self.videoResolutionY=self.validResolutions[0][1]

            except ValueError:
                self.videoResolutionX=self.validResolutions[0][0]
                self.videoResolutionY=self.validResolutions[0][1]

        def nextResolution(self):
            try:
                val=self.validResolutions.index((self.videoResolutionX,self.videoResolutionY))

                if val>-1 and val<(len(self.validResolutions)-1):
                    self.videoResolutionX=self.validResolutions[val+1][0]
                    self.videoResolutionY=self.validResolutions[val+1][1]
                else:
                    self.videoResolutionX=self.validResolutions[len(self.validResolutions)-1][0]
                    self.videoResolutionY=self.validResolutions[len(self.validResolutions)-1][1]
            except ValueError:
                self.videoResolutionX=self.validResolutions[len(self.validResolutions)-1][0]
                self.videoResolutionY=self.validResolutions[len(self.validResolutions)-1][1]

        def nextFullScreen(self):
            if self.videoFullScreen:
                self.videoFullScreen=False
            else:
                self.videoFullScreen=True
        def prevFullScreen(self):
            self.nextFullScreen()

        def lessVolumeGeneral(self):
            self.volumeGeneral=(self.volumeGeneral-1)%11
        def lessVolumeMusic(self):
            self.volumeMusic=(self.volumeMusic-1)%11
        def lessVolumeSound(self):
            self.volumeSound=(self.volumeSound-1)%11
        def moreVolumeGeneral(self):
            self.volumeGeneral=(self.volumeGeneral+1)%11
        def moreVolumeMusic(self):
            self.volumeMusic=(self.volumeMusic+1)%11
        def moreVolumeSound(self):
            self.volumeSound=(self.volumeSound+1)%11





    class Control (object):
        
        def __init__(self):

            self.up=pygame.K_UP
            self.down=pygame.K_DOWN
            self.left=pygame.K_LEFT
            self.right=pygame.K_RIGHT
            self.button1=pygame.K_z
            self.button2=pygame.K_x
            self.button3=pygame.K_a
            self.button4=pygame.K_s


        def export(self,number):

            number=number%5
            if number==0:
                number=1

            string="\n\t\t\t\t<up>"+str(self.up)+"</up>"+"\n\t\t\t\t<down>"+str(self.down)+"</down>"
            string=string+"\n\t\t\t\t<left>"+str(self.left)+"</left>"+"\n\t\t\t\t<right>"+str(self.right)+"</right>"
            string=string+"\n\t\t\t\t<button1>"+str(self.button1)+"</button1>"+"\n\t\t\t\t<button2>"+str(self.button2)+"</button2>"
            string=string+"\n\t\t\t\t<button3>"+str(self.button3)+"</button3>"+"\n\t\t\t\t<button4>"+str(self.button4)+"</button4>"

            return "\n\t\t\t<P"+str(number)+">"+string+"\n\t\t\t</P"+str(number)+">"

        def validateControl(self):
            self.up=self.validateButton(self.up)
            self.down=self.validateButton(self.down)
            self.left=self.validateButton(self.left)
            self.right=self.validateButton(self.right)
            self.button1=self.validateButton(self.button1)
            self.button2=self.validateButton(self.button2)
            self.button3=self.validateButton(self.button3)
            self.button4=self.validateButton(self.button4)


        def validateButton(self, button):
            if type(button)<>int:
                try:
                    return int(button)
                except ValueError:
                    return 0
            else:
                return button


        def getButtonList(self):
            lista=[pygame.key.name(self.up),pygame.key.name(self.down),pygame.key.name(self.left),pygame.key.name(self.right)]
            lista=lista+[pygame.key.name(self.button1),pygame.key.name(self.button2),pygame.key.name(self.button3),pygame.key.name(self.button4)]
            return lista



# -----------TODO: PRUEBAS UNITARIAS --------------

if __name__=="__main__":

    print "\n*************TESTING**************"
    c=ConfigHandler()


    c.actualConfig=ConfigHandler.Config()

    """

    pygame.init()
    print "\n-----------GETKEY----------"
    print "   Configuracion por defecto, boton up del 1er jugador"
    print c.getKey(1,"UP")
    print c.getKey(1,"UP")


    print "\n-----------IMPORTCONFIG----------"

    print "   Botones1 de los mandos de las configuraciones myconfig(basica, xml vacio) y standard(Z del XML)"



    print "   Import Config Erroneo, archivo inexistente:"
    c.importConfig("../Escritorio/Arcadia Temp/entrada.xml")

    print "   Import Config Correcto"
    c.importConfig("config/general/config.xml")


    for e in c.configs:
        print e.name
        for i in e.controls:
            print i.button1

    """

    print "\n-----------SETACTUALCONFIG----------"

    print "   Configuracion Actual, la estandar"
    c.setActualConfigByName("standard")
    print c.actualConfig.name


    """
    print "   Configuracion de nombre standard, boton up del 1er jugador"
    print c.getKey(1,"UP")
    print c.getKey(1,"UP")


    print "   Configuraciones Disponibles:"
    print c.getNameConfigs()


    print "\n-----------Prueba EXPORT------------\n"

    print "   Archivo original"

    content=FileManager.fileReader("config/general/config.xml")
    stri=''
    for e in content:
        stri=stri+e
    print stri

    print "    Resultado de validacion"
    string= c.exportFileConfig([c.actualConfig])

    print string

 
    print c.actualConfig.validateConfig()
    string= c.exportFileConfig([c.actualConfig])


    print "--------------VALIDADO:"
    print string


    FileManager.fileWriter("../Escritorio/Arcadia Temp/salida.xml",string)

    
    #string=c.exportFileConfig([c.actualConfig]+c.configs=[])
    #FileManager.fileWriter(config/general/configresult.xml,string) 


    pygame.quit()
    """




