###############################################################################
## Arcadia Project
##
## Copyright (C) 2009-2010 Arcadia Project arcadia.videogame@gmail.com http://www.arcadiaproject.org/
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###############################################################################

import pygame
from pygame.locals import *
import sys, os, string
from SimpleSurface import SimpleSurface

class FontManager(object):
    """clase para trabajar con texto en el juego"""

    def __init__(self, size = 12):
        self.font = pygame.font.SysFont("Arial", size)
        self.size = size
        self.surface = None

    def selectSystemFont(self, font, size):
        self.font = pygame.font.SysFont(font, size)

    def selectCustomFont(self, font, size):
        self.font = pygame.font.Font(os.path.join('resources/fonts', font + '.ttf'), size)

    def renderText(self, text, left, top, color = (0,0,0), halign = "left", valign = "top"):
        #renderiza el texto en una surface para luego pintarla con el render
        self.surface = SimpleSurface(self.font.render(text, True, color), left, top)
        if halign == "center":
            centeredx = self.surface.max_rect().left - self.surface.max_rect().width / 2
            self.surface.rect.left = centeredx

        if valign == "center":
            centeredy = self.surface.max_rect().top - self.surface.max_rect().height / 2
            self.surface.rect.top = centeredy

        if halign == "right":
            dx = left - self.surface.max_rect().width
            self.surface.rect.left = dx

        if valign == "bottom":
            dy = top - self.surface.max_rect().height
            self.surface.rect.top = dy
        return self.surface

    def renderParagraph(self, text, left, top, width, height, color = (0,0,0), halign = "left", valign = "top"):
        """metodo que se encarga de dividir en lineas el texto pasado por parametro teniendo en cuenta que no corte las palabras"""
        text_length = len(text)
        words_list = string.split(text, ' ') #dividimos en palabras
        lines = []
        line = ""
        for word in words_list:
            #calculamos el tamanio de la palabra en pixels
           
            word_width = len(word) * self.size
            if (len(line) * self.size) + word_width < width:
                line = line + word + " "

            else:
                lines.append(line)
                line = word + " "

        if len(line) > 0 :
            lines.append(line)
 

        surface = pygame.Surface((width, height), SRCALPHA)
        
        lineCounter = 0
        for line in lines:
            surface.blit(self.renderText(line, left, top, color, halign, valign).getSurface(), (width * 0.01, lineCounter * 2 *self.size))
            lineCounter += 1

        rect = surface.get_rect()
        surface = SimpleSurface(surface)
        surface.setRect(rect.left, rect.top)
        return surface
            
            
        

    def getSurface(self):
        return self.surface.getSurface()
