###############################################################################
## Arcadia Project
##
## Copyright (C) 2009-2010 Arcadia Project arcadia.videogame@gmail.com http://www.arcadiaproject.org/
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###############################################################################

import pygame
import FileManager

class SimpleSurface(object):
    """clase simple para manejar surfaces sin animacion"""

    def __init__(self, surface, left = 0, top = 0, halign="left", valign="top"):
        if isinstance(surface, pygame.Surface):
            self.surface = surface
        else:
            self.surface = pygame.image.load(FileManager.locationFixer(surface)).convert_alpha()

        self.rect = self.surface.get_rect()
        self.printable = True
        self.rect.left = left
        self.rect.top = top

        if halign == "center":
            centeredx = self.rect.left - self.rect.width * 0.5
            self.rect.left = centeredx

        if valign == "center":
            centeredy = self.rect.top - self.rect.height * 0.5
            self.rect.top = centeredy

        if halign == "right":
            dx = left - self.rect.width
            self.rect.left = dx

        if valign == "bottom":
            dy = top - self.rect.height
            self.rect.top = dy

    def setRect(self, left, top, halign="left", valign="top"):
        
        self.rect.left = left
        self.rect.top = top

        if halign == "center":
            centeredx = left - int(self.rect.width * 0.5)
            self.rect.left = centeredx

        if valign == "center":
            centeredy = top - int(self.rect.height * 0.5)
            self.rect.top = centeredy

        if halign == "right":
            dx = left - self.rect.width
            self.rect.left = dx

        if valign == "bottom":
            dy = top - self.rect.height
            self.rect.top = dy


    def setName(self, name):
        self.name = name

    def getName(self):
        return self.name

    def getSurface(self):
        return self.surface

    def setSurface(self, surface):
        self.surface = surface
        self.rect.width = surface.get_rect().width
        self.rect.height = surface.get_rect().height

    def max_rect(self):
        return self.rect
    
    def isPrintable(self):
        return self.printable
    
    def hide(self):
        self.printable = False

    def show(self):
        self.printable = True

    def move(self, x, y):
        self.rect.left += x
        self.rect.top += y

